// Plugin.cpp : Defines the entry point for the DLL application.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Plugin.h"

char* g_pBuffer = NULL;

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			{
				if (g_pBuffer)
				{
					delete [] g_pBuffer;
					g_pBuffer = NULL;
				}
			}
			break;
    }
    return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//
// CS_GetDescription()
//
///////////////////////////////////////////////////////////////////////////////
PLUGIN_API LPCSTR CS_GetDescription(void)
{
	// 'runforall' values "yes" or "no"
	static char* t_Buffer = {
		"<plugins>"\
			"<plugininfo>"\
				"<menuname>Remove Images</menuname>"\
				"<menuhelp>Remove all images from selected collectible</menuhelp>"\
				"<methodname>CS_Method1</methodname>"\
				"<runforall>no</runforall>"\
			"</plugininfo>"\
			"<plugininfo>"\
				"<menuname>Remove All Images</menuname>"\
				"<menuhelp>Remove all images from all collectibles in the collection</menuhelp>"\
				"<methodname>CS_Method1</methodname>"\
				"<runforall>yes</runforall>"\
			"</plugininfo>"\
		"</plugins>"
	};

	return t_Buffer;
}

///////////////////////////////////////////////////////////////////////////////
//
// CS_Method1()
//
///////////////////////////////////////////////////////////////////////////////
PLUGIN_API LPCSTR CS_Method1(LPCSTR a_Collectible, size_t a_CollectibleLength)
{
	// make 
	char* t_pBuf = new char[a_CollectibleLength + 1];
	if (! t_pBuf)
		return NULL;
	memcpy(t_pBuf, a_Collectible, sizeof(char)*a_CollectibleLength+1);

	pug::xml_parser xml;
	if (! xml.parse(t_pBuf, pug::parse_default))
	{
		// error
		delete t_pBuf;
		return NULL;
	}

	pug::xml_node & t_Root = xml.document();

	// for each collection
	for (unsigned int i = 0; i < t_Root.children(); i++)
	{
		pug::xml_node & t_CollectionItem = t_Root.child(i);

		if (stricmp("Collection", t_CollectionItem.name()))
			continue;

		// for each collectible in the current collection
		for (unsigned int g = 0; g < t_CollectionItem.children(); g++)
		{
			pug::xml_node & t_ItemInfo = t_CollectionItem.child(g);

//				if (! stricmp("coin", t_ItemInfo.name()) ||
//					! stricmp("banknote", t_ItemInfo.name()) ||
//					! stricmp("poster", t_ItemInfo.name()) ||
//					! stricmp("book", t_ItemInfo.name()) ||
//					! stricmp("stamp", t_ItemInfo.name()) )
			{
				// for all supported collectibles

				for (pug::xml_node::xml_node_iterator it=t_CollectionItem.begin(); it!=t_CollectionItem.end(); ++it)
				{
					if (! stricmp("images", (*it).name()))
					{
						t_CollectionItem.children_erase(it);
						break;
					}
				}
			}
		}
	}

	LPCSTR t_pReturn = PackToXML(t_Root, a_CollectibleLength);
	delete t_pBuf;
	return t_pReturn;
}

///////////////////////////////////////////////////////////////////////////////
//
// PackToXML()
//
///////////////////////////////////////////////////////////////////////////////
LPCSTR PackToXML(pug::xml_node & t_Root, size_t t_nSize)
{
	if (g_pBuffer)
	{
		delete [] g_pBuffer;
		g_pBuffer = NULL;
	}

	g_pBuffer = new char[t_nSize * 2];
	if (! g_pBuffer)
	{
		return NULL;
	}

	*g_pBuffer = 0;
	PackToXMLNode(t_Root, g_pBuffer);

	return g_pBuffer;
}

///////////////////////////////////////////////////////////////////////////////
//
// PackToXMLNode()
//
// Recursive method.
//
///////////////////////////////////////////////////////////////////////////////
void PackToXMLNode(pug::xml_node & t_Root, char* t_pBuffer)
{
	if (*t_Root.name())
	{
		strcat(t_pBuffer, "<");
		strcat(t_pBuffer, t_Root.name());
		strcat(t_pBuffer, ">");
	}

	if (t_Root.children())
	{
		for (unsigned int i = 0; i < t_Root.children(); i++)
		{
			pug::xml_node & t_Child = t_Root.child(i);
			PackToXMLNode(t_Child, t_pBuffer);
		}
	}
	else
	{
		strcat(t_pBuffer, t_Root.value());
	}

	if (*t_Root.name())
	{
		strcat(t_pBuffer, "</");
//		strcat(t_pBuffer, &t_Root.name()[1]);
		strcat(t_pBuffer, t_Root.name());
		strcat(t_pBuffer, ">");
	}
}
